@@ -3,7 +3,9 @@
 // http://opensource.org/licenses/MIT>. This file may not be copied, modified,
 // or distributed except according to those terms.
 
-//! Space-efficient optional values
+//! # Space-efficient optional values
+//! 
+//! ## Booleans
 //!
 //! Type `OptionBool` represents an optional boolean value, similar to
 //! `Option<bool>`. Most function implementations are similar or equal.
@@ -14,15 +16,26 @@
 //!
 //! The `OptionBool` type is expected to require only 1 byte of storage:
 //!
-//! ```
+//! ```rust
 //! assert!(1 == std::mem::size_of::<optional::OptionBool>());
 //! ```
+//! However, since this crate was originally authored, [improvements in the
+//! compiler](https://users.rust-lang.org/t/list-of-fundamental-crates/17806/5?u=ryan)
+//! have built in this optimization for `Option<bool>` as well. The 
+//! `OptionBool` type remains however because it [may still perform faster in
+//! some bench marks](https://github.com/llogiq/optional/issues/33). 
+//! 
+//! ```rust
+//! assert!(1 == std::mem::size_of::<Option<bool>>());
+//! ```
 //!
+//! ## Any type can be optional
+//! 
 //! Then there is the `Optioned<T>` type which wraps a type `T` as an optional
 //! value of `T` where one particular value represents None. `Optioned<T>`
 //! requires the exact same space as T:
 //!
-//! ```
+//! ```rust
 //! assert!(std::mem::size_of::<optional::Optioned<i64>>() ==
 //!     std::mem::size_of::<i64>());
 //! assert!(std::mem::size_of::<optional::Optioned<f32>>() ==
@@ -38,6 +51,7 @@
 //!
 //! Using Optioned for your own types is as simple as implementing `Noned` for
 //! your type, provided that your type is already Copy and Sized.
+//! 
 
 #![deny(missing_docs)]
 
@@ -1311,7 +1325,13 @@ where
     where
         S: serde::Serializer,
     {
-        self.as_option().serialize(serializer)
+        let opt = if self.value.is_none() {
+            Option::None
+        } else {
+            Option::Some(self.value)
+        };
+
+        opt.serialize(serializer)
     }
 }
 
@@ -1370,8 +1390,31 @@ impl<T: Noned + Copy> Optioned<T> {
         }
     }
 
+    /// Convenience funtion to convert an `Optioned` into an `Option`.
+    /// 
+    /// Techinically this is possible with the `Into<Option<T>>` implementation in this crate. 
+    /// However, experience has shown that there are many cases where the compiler isn't able to 
+    /// infer the correct type for the generic parameter and issues an error. As a workaround you 
+    /// can use `Into::<Option<T>>::into(...)` or `.map(|inner| inner)` which will both work.
+    /// The first is verbose and the second makes the intention less clear, so this method is 
+    /// provided as a convenience.
+    /// 
+    /// ```rust
+    ///# use optional::some;
+    /// if let Some(val) = some(15).into_option() {
+    ///     println!("val = {}", val);
+    /// }
+    /// ```
+    /// 
+    /// The following example will fail to compile because of type inference.
+    /// ```compile_fail
+    /// # use optional::some;
+    /// if let Some(val) = some(15).into() {
+    ///     println!("val = {}", val);
+    /// }
+    /// ```
     #[inline]
-    fn as_option(&self) -> Option<T> {
+    pub fn into_option(self) -> Option<T> {
         if self.value.is_none() {
             Option::None
         } else {
@@ -1768,7 +1811,7 @@ impl<T: Noned + Copy> Optioned<T> {
     /// ```
     #[inline]
     pub fn take(&mut self) -> Option<T> {
-        mem::replace(self, Self::none()).as_option()
+        mem::replace(self, Self::none()).into_option()
     }
 
     /// Return a possibly empty slice over the contained value, if any.
@@ -1897,7 +1940,14 @@ impl<T: Noned + Copy> From<Option<T>> for Optioned<T> {
 impl<T: Noned + Copy> Into<Option<T>> for Optioned<T> {
     #[inline]
     fn into(self) -> Option<T> {
-        self.as_option()
+        self.into_option()
+    }
+}
+
+impl<'a, T: Noned + Copy> Into<Option<T>> for &'a Optioned<T> {
+    #[inline]
+    fn into(self) -> Option<T> {
+        self.into_option()
     }
 }
 
